       IDENTIFICATION DIVISION.
       PROGRAM-ID.    BSDCOBBC.
       AUTHOR.        CSI.
       DATE-WRITTEN.  NOV 2007.
      ******************************************************************
      *                                                                *
      *    THIS IS A SAMPLE BSD/C CLIENT PROGRAM                       *
      *                                                                *
      *    THE FOLLOWING BSD/C FUNCTIONS ARE PERFORMED:                *
      *                                                                *
      *    - SETERRD() REQUEST TO SPECIFY FIELDS FOR ERRNO AND RETCD   *
      *    - SOCKET() REQUEST TO GET A SOCKET NUMBER                   *
      *    - CONNECT() REQUEST TO CONNECT TO A SERVER                  *
      *    - SEND() REQUEST TO SEND DATA TO THE SERVER                 *
      *    - RECEIVE() REQUEST TO GET RESPONSE FROM THE SERVER         *
      *    - CLOSE() REQUEST TO TERMINATE CONNECTION TO SERVER         *
      *      AND RELEASE THE SOCKET.                                   *
      *                                                                *
      *   SEVERAL 'DISPLAY' COMMANDS ARE INCLUDED JUST FOR             *
      *   DEMONSTRATION PURPOSES, YOU WOULD NORMALLY NOT INCLUDE MOST  *
      *   OF THEM.                                                     *
      *                                                                *
      ******************************************************************
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       DATA DIVISION.
       WORKING-STORAGE SECTION.
 
       01 FILLER PIC X(24) VALUE 'BSDCOBBC-WORKING-STORAGE'.
       01 TCPIP-CONSTANTS.
      *       SOCKET.H AF_INET
          05  SOCKET-DOMAIN            PIC S9(8) COMP VALUE +2.                 
      *       SOCKET.H SOCK_STREAM
          05  SOCKET-TYPE              PIC S9(8) COMP VALUE +1.                 
      *       SOCKET.H IPPROTO_TCP
          05  SOCKET-PROTOCOL          PIC S9(8) COMP VALUE +2.                 
 
       01 TCPIP-FIELDS.
          05  TCPIP-SOCKET             PIC S9(8) COMP VALUE +0.                 
          05  TCPIP-ERRNO              PIC S9(8) COMP VALUE +0.                 
          05  TCPIP-RETCODE            PIC S9(8) COMP VALUE +0.                 
          05  TCPIP-SENDBUFF-LEN       PIC S9(8) COMP VALUE +80.                
          05  TCPIP-SENDBUFF           PIC X(80).
          05  TCPIP-RECVBUFF-LEN       PIC S9(8) COMP VALUE +80.                
          05  TCPIP-RECVBUFF           PIC X(80).
          05  TCPIP-LEN                PIC S9(4) COMP VALUE +0.                 
          05  TCPIP-SOCKET-ACQ         PIC X          VALUE 'N'.
          05  TCPIP-STATUS             PIC X          VALUE SPACE.
 
       01 TCPIP-SOCKADDR.
          05  FILLER                   PIC S9(4) COMP VALUE +2.                 
          05  SOCKADDR-PORT            PIC S9(4) COMP VALUE +0.                 
          05  SOCKADDR-IPADDR          PIC X(4)  VALUE LOW-VALUES.              
          05  FILLER                   PIC X(8)  VALUE LOW-VALUES.
       01 TCPIP-SOCKADDR-LEN           PIC S9(8) COMP VALUE +16.                
 
       01 DISPLAY-RC                   PIC ZZZZZZZ9-.
       01 IPWORK.
          05 IP-1                      PIC 9(4) COMP.
          05 IP-2                      PIC 9(4) COMP.
          05 IP-3                      PIC 9(4) COMP.
          05 IP-4                      PIC 9(4) COMP.
      ******************************************************************
      *                                                                *
      *    START OF PROCEDURE DIVISION.                                *
      *                                                                *
      ******************************************************************
 
       PROCEDURE DIVISION.
           DISPLAY 'BSDCOBBC ENTRY'.
 
      ******************************************************************
      *
      * * BSD-C SETERRD() CALL INVOKES IPNRERRD...
      *
      *   THIS FUNCTION SETS 'DEFAULT' LOCATIONS FOR RETURNING THE
      *   TCPIP ERRNO AND RETCODE FIELDS
 
           DISPLAY 'BSDCOBBC CALLING IPNRERRD'.
           CALL 'IPNRERRD' USING TCPIP-ERRNO, TCPIP-RETCODE.
           MOVE RETURN-CODE TO DISPLAY-RC.
           DISPLAY 'BSDCOBBC RC=' DISPLAY-RC
                        ' RETCODE=' TCPIP-RETCODE
                        ' ERRNO=' TCPIP-ERRNO.
 
      ******************************************************************
      *
      * * BSD-C SOCKET() CALL INVOKES IPNRSOCK...
      *
      *   THIS FUNCTION RESERVES A 'SOCKET NUMBER' TO BE USED
      *   FOR ESTABLISHING A CONNECTION TO A SERVER.
      *
      *   A RETURN-CODE LESS THAN ZERO INDICATES A FAILURE
 
           DISPLAY 'BSDCOBBC CALLING IPNRSOCK'.
           CALL 'IPNRSOCK' USING SOCKET-DOMAIN,
                                 SOCKET-TYPE,
                                 SOCKET-PROTOCOL.
           MOVE RETURN-CODE TO TCPIP-SOCKET.
           MOVE RETURN-CODE TO DISPLAY-RC.
           DISPLAY 'BSDCOBBC RC=' DISPLAY-RC
                        ' RETCODE=' TCPIP-RETCODE
                        ' ERRNO=' TCPIP-ERRNO.
           IF RETURN-CODE IS LESS THAN ZERO
               MOVE 'X' TO TCPIP-STATUS
           ELSE
               MOVE 'Y' TO TCPIP-SOCKET-ACQ
           END-IF.
 
      ******************************************************************
      *
      * * BSD-C CONNECT() CALL INVOKES IPNRCONN...
      * * THIS WILL CAUSE A ACTIVE OPEN TO BE ISSUED...
      * * CONNECT TO IPADDR 172.27.34.5, PORT 6045
      *
      *   A RETURN-CODE LESS THAN ZERO INDICATES A FAILURE
 
           MOVE  172 TO IP-1.                                                  
           MOVE  27  TO IP-2.                                                  
           MOVE  34  TO IP-3.                                                   
           MOVE  5   TO IP-4.                                                   
           MOVE IPWORK(2:1) TO SOCKADDR-IPADDR(1:1).                            
           MOVE IPWORK(4:1) TO SOCKADDR-IPADDR(2:1).                            
           MOVE IPWORK(6:1) TO SOCKADDR-IPADDR(3:1).                            
           MOVE IPWORK(8:1) TO SOCKADDR-IPADDR(4:1).                            
           MOVE 6045        TO SOCKADDR-PORT.
                                                                                
           IF TCPIP-STATUS = SPACE
               DISPLAY 'BSDCOBBC CALLING IPNRCONN, SOCKET='
                       TCPIP-SOCKET
               CALL 'IPNRCONN' USING TCPIP-SOCKET,
                                     TCPIP-SOCKADDR,
                                     TCPIP-SOCKADDR-LEN
               MOVE RETURN-CODE TO DISPLAY-RC
               DISPLAY 'BSDCOBBC RC=' DISPLAY-RC
                       ' RETCODE=' TCPIP-RETCODE
                       ' ERRNO=' TCPIP-ERRNO
               IF RETURN-CODE IS LESS THAN ZERO
                   MOVE 'X' TO TCPIP-STATUS
               END-IF
           END-IF.
      *
           MOVE 'HARTBEAT' TO TCPIP-SENDBUFF.
           DISPLAY TCPIP-SENDBUFF.
           MOVE X'15'      TO TCPIP-SENDBUFF(73:1).
 
      ******************************************************************
      *
      * * BSD-C SEND FUNCTION CALL...
      *
           IF TCPIP-STATUS = SPACE
               DISPLAY 'BSDCOBBC CALLING IPNRSEND, SOCKET='
                       TCPIP-SOCKET
               CALL 'IPNRSEND' USING TCPIP-SOCKET,
                                     TCPIP-SENDBUFF,
                                     TCPIP-SENDBUFF-LEN
               MOVE RETURN-CODE TO DISPLAY-RC
               DISPLAY 'BSDCOBBC RC=' DISPLAY-RC
                       ' RETCODE=' TCPIP-RETCODE
                       ' ERRNO=' TCPIP-ERRNO
               IF RETURN-CODE IS LESS THAN ZERO
                   MOVE 'X' TO TCPIP-STATUS
               END-IF
           END-IF.
 
      ******************************************************************
      *
      * * BSD-C RECV FUNCTION CALL...
      *
      *    THIS SAMPLE ASSUMES THAT THE COMPLETE INPUT MESSAGE IS
      *    FOUND IN A SINGLE RECEIVE. IN REALITY, YOU WILL NEED TO
      *    DETERMINE IF THE ENTIRE MESSAGE HAS BEEN RECEIVED, AND
      *    WILL MOST LIKELY NEED TO DO SOME FORM OF RECEIVE LOOPING
      *    TO GET THE ENTIRE MESSAGE.
      *
           IF TCPIP-STATUS = SPACE
               DISPLAY 'BSDCOBBC CALLING IPNRRECV, SOCKET='
                       TCPIP-SOCKET
               CALL 'IPNRRECV' USING TCPIP-SOCKET,
                                     TCPIP-RECVBUFF,
                                     TCPIP-RECVBUFF-LEN
               MOVE RETURN-CODE TO TCPIP-LEN
               MOVE RETURN-CODE TO DISPLAY-RC
               DISPLAY 'BSDCOBBC RC=' DISPLAY-RC
                       ' RETCODE=' TCPIP-RETCODE
                       ' ERRNO=' TCPIP-ERRNO
               IF RETURN-CODE IS LESS THAN ZERO
                   MOVE 'X' TO TCPIP-STATUS
               END-IF
               DISPLAY TCPIP-RECVBUFF(1:TCPIP-LEN)
           END-IF.
 
      ******************************************************************
      *
      * * BSD-C CLOSE FUNCTION CALL...
      *
           IF TCPIP-SOCKET-ACQ = 'Y'
               DISPLAY 'BSDCOBBC CALLING IPNRCLOS, SOCKET='
                       TCPIP-SOCKET
               CALL 'IPNRCLOS' USING TCPIP-SOCKET
               MOVE RETURN-CODE TO DISPLAY-RC
               DISPLAY 'BSDCOBBC RC=' DISPLAY-RC
                       ' RETCODE=' TCPIP-RETCODE
                       ' ERRNO=' TCPIP-ERRNO
           END-IF.
      *
           DISPLAY 'BSDCOBBC EXIT'.
           MOVE ZERO TO RETURN-CODE.
           STOP RUN.
