       IDENTIFICATION DIVISION.
       PROGRAM-ID.    BSDCOBCS.
       AUTHOR.        CSI.
       DATE-WRITTEN.  NOV 2007.
      ******************************************************************
      *                                                                *
      * THIS SOURCE MODULE IS PART OF A SET OF MODULES THAT REPRESENT  *
      * A SAMPLE BSD/C SERVER THAT RUNS IN CICS.                       *
      *                                                                *
      * THE SET OF MODULES INCLUDE THE FOLLOWING:                      *
      *                                                                *
      *  BSDCOBCL - MAIN LISTEN TASK                                   *
      *  BSDCOBCS - SERVER TASK THAT PROCESSES THE CONNECTED TASK(S)   *
      *  BSDCOBCX - TASK TO START/STOP THE LISTEN AND SERVER TASKS     *
      *  BSDCOBCT - TABLE TO CONTAIN COMMON FIELDS                     *
      *                                                                *
      *  THE REQUIRED CICS DEFINITIONS CAN BE FOUND IN MODULE BSDCOBCL *
      *                                                                *
      *  BSDCOBCS                                                      *
      *                                                                *
      *  THIS PROGRAM PERFORMS THE PROCESSING OF THE CLIENT REQUEST.   *
      *                                                                *
      *  IT IS 'STARTED' BY BSDCOBCL WHEN A CONNECTION IS MADE.        *
      *                                                                *
      *  THIS PROGRAM THAN RETURNS TO THE LISTEN STATE WAITING FOR     *
      *  ANOTHER CONNECTION.                                           *
      *                                                                *
      *  THERE ARE MANY DISPLAY COMMANDS IN THIS SAMPLE PROGRAM THAT   *
      *  WOULD NOT NORMALLY BE IN A PRODUCTION SYSTEM. THEY ARE HERE   *
      *  TO HELP ILLUSTRATE WHAT THE PROGRAM IS DOING.                 *
      *                                                                *
      ******************************************************************
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       DATA DIVISION.
       WORKING-STORAGE SECTION.
 
       01  WS-EYE PIC X(24) VALUE 'BSDCOBCS-WORKING-STORAGE'.
       01  TCPIP-CONSTANTS.
           05  MAX-SOCKETS              PIC S9(8) COMP VALUE +128.
 
       01  TCPIP-FIELDS.
           05  TCPIP-TIMEOUT.
               10  TCPIP-TIMEOUT-SECS   PIC S9(8) COMP VALUE +30.
               10  TCPIP-TIMEOUT-USECS  PIC S9(8) COMP VALUE +0.
           05  TCPIP-ACCEPT-SOCKET      PIC S9(8) COMP VALUE +0.                N19990622 16:54:09
           05  TCPIP-SENDBUFF-LEN       PIC S9(8) COMP VALUE +80.               N19990622 16:54:09
           05  TCPIP-SENDBUFF           PIC X(80).
           05  TCPIP-RECVBUFF-LEN       PIC S9(8) COMP VALUE +80.               N19990622 16:54:09
           05  TCPIP-RECVBUFF           PIC X(80).
           05  TCPIP-LEN                PIC S9(4) COMP VALUE +0.                N19990622 16:54:09
           05  TCPIP-STATUS             PIC X          VALUE SPACE.
           05  TCPIP-MASK-READ          PIC X(16) VALUE LOW-VALUES.
           05  TCPIP-MASK-WRITE         PIC X(16) VALUE LOW-VALUES.
           05  TCPIP-MASK-EXCP          PIC X(16) VALUE LOW-VALUES.
 
       01  TCPIP-DUMMY-ECB              PIC X(4)  VALUE LOW-VALUES.
 
       01  TCPIP-SOCKADDR.
           05  FILLER                   PIC S9(4) COMP VALUE +2.                N19990622 16:54:09
           05  SOCKADDR-PORT            PIC S9(4) COMP VALUE +6045.             N19990622 16:54:09
           05  SOCKADDR-IPADDR          PIC X(4).                               N19990622 16:54:09
           05  FILLER                   PIC X(8)  VALUE LOW-VALUES.
       01  TCPIP-SOCKADDR-LEN           PIC S9(8) COMP VALUE +16.               N19990622 16:54:09
 
       01  DISPLAY-RC                   PIC ZZZZZZZ9-.
 
       01  BIT-WORK-AREAS.
           05 BIT-BITS                  PIC X(8) VALUE
                                          X'8040201008040201'.
           05 BIT-MASK-OFFSET           PIC S9(4) COMP VALUE +0.
           05 BIT-BIT-OFFSET            PIC S9(4) COMP VALUE +0.
 
       LINKAGE SECTION.
 
      ******************************************************************
      *                                                                *
      *    BSDCOBCT TABLE  - LOADED FROM BSDCOBCT                      *
      *                                                                *
      *    THE FORMAT OF THIS TABLE IS KNOWN BY PROGRAMS:              *
      *                                                                *
      *    BSDCOBCL - SAMPLE COBOL BSD/C SERVER LISTEN TASK            *
      *    BSDCOBCS - SAMPLE COBOL BSD/C SERVER PROCESS TASK           *
      *    BSDCOBCX - SAMPLE COBOL BSD/C SERVER TERMINATION TASK       *
      *                                                                *
      ******************************************************************
 
       01  BSDCOBCT-TABLE.
           05  BSDCOBCT-EYECATCHER          PIC X(16).
           05  BSDCOBCT-ERRNO               PIC S9(8) COMP.                     N19990622 16:54:09
           05  BSDCOBCT-RETCODE             PIC S9(8) COMP.                     N19990622 16:54:09
      *    ECB MUST START ON FULL-WORD
           05  BSDCOBCT-TERM-ECB            PIC X(4).
           05  BSDCOBCT-TERM-FLAG           PIC X.
 
      ******************************************************************
      *                                                                *
      *    START OF PROCEDURE DIVISION.                                *
      *                                                                *
      ******************************************************************
 
       PROCEDURE DIVISION.
           DISPLAY 'BSDCOBCS ENTRY'.
 
      ******************************************************************
      *
      *    LOAD COMMON CONTROL TABLE
      *
           DISPLAY 'BSDCOBCS LOADING BSDCOBCT'.
 
           EXEC CICS LOAD PROGRAM('BSDCOBCT') NOHANDLE
                              SET(ADDRESS OF BSDCOBCT-TABLE) END-EXEC.
 
           IF EIBRESP IS NOT = ZERO
               DISPLAY 'BSDCOBCS LOAD OF BSDCOBCT FAILED, RC='
                       EIBRESP
               GO TO 90000-EXIT-CICS
           ELSE
               DISPLAY BSDCOBCT-EYECATCHER
           END-IF.
 
           IF BSDCOBCT-TERM-FLAG = 'X'
               DISPLAY 'BSDCOBCS TERMINATE REQUESTED'
               GO TO 90000-EXIT-CICS
           END-IF.
 
      ******************************************************************
      *
      * * BSD-C SETERRD() CALL INVOKES IPNRERRD...
      *
      *   THIS FUNCTION SETS 'DEFAULT' LOCATIONS FOR RETURNING THE
      *   TCPIP ERRNO AND RETCODE FIELDS
      *
           DISPLAY 'BSDCOBCS CALLING IPNRERRD'.
           CALL 'IPNRERRD' USING BSDCOBCT-ERRNO, BSDCOBCT-RETCODE.
           MOVE RETURN-CODE TO DISPLAY-RC.
           DISPLAY 'BSDCOBCS RC=' DISPLAY-RC
                        ' RETCODE=' BSDCOBCT-RETCODE
                        ' ERRNO=' BSDCOBCT-ERRNO
                        ' EIBRESP=' EIBRESP ' EIBRESP2=' EIBRESP2.
 
      ******************************************************************
      *
      * * GET SOCKET NUMBER FOR THIS SERVER REQUEST
      *
      *    PASSED FROM BSDCOBCL
      *
           EXEC CICS RETRIEVE INTO(TCPIP-ACCEPT-SOCKET)
                     NOHANDLE END-EXEC.
 
      ******************************************************************
      *                                                                *
      *   LOOP HERE TO WAIT FOR EACH NEW INCOMING DATA                 *
      *                                                                *
      ******************************************************************
 
       30000-SELECTEX.
 
           IF BSDCOBCT-TERM-FLAG = 'X'
               DISPLAY 'BSDCOBCS TERMINATE REQUESTED'
               GO TO 50000-CLOSE-TCP
           END-IF.
 
      ******************************************************************
      *
      * * BSD-C SELECTEX() FUNCTION CALL...
      * * THIS WILL ISSUE A WAIT...BSD-C CALLS THIS BLOCKING.
      * * UNTIL DATA IS READY TO RECEIVE...
      *
      *    FIRST SET BIT IN READ MASK FOR ACCEPT SOCKET NUMBER
      *
           DIVIDE TCPIP-ACCEPT-SOCKET BY 8 GIVING BIT-MASK-OFFSET
                                 REMAINDER BIT-BIT-OFFSET.
           ADD +1 TO BIT-MASK-OFFSET.
           ADD +1 TO BIT-BIT-OFFSET.
           MOVE LOW-VALUES TO TCPIP-MASK-READ.
           MOVE BIT-BITS (BIT-BIT-OFFSET:1) TO
                TCPIP-MASK-READ (BIT-MASK-OFFSET:1).
           DISPLAY 'BSDCOBCS MOFF=' BIT-MASK-OFFSET
                           ' BOFF=' BIT-BIT-OFFSET
                           ' MASK=' TCPIP-MASK-READ.
 
           DISPLAY 'BSDCOBCS CALLING IPNRSELX'.
           CALL 'IPNRSELX' USING MAX-SOCKETS,
                                 TCPIP-MASK-READ,
                                 TCPIP-MASK-WRITE,
                                 TCPIP-MASK-EXCP,
                                 TCPIP-TIMEOUT,
                                 BSDCOBCT-TERM-ECB,
                                 TCPIP-DUMMY-ECB.
           MOVE RETURN-CODE TO DISPLAY-RC.
           DISPLAY 'BSDCOBCS RC=' DISPLAY-RC
                   ' RETCODE=' BSDCOBCT-RETCODE
                   ' ERRNO=' BSDCOBCT-ERRNO
                   ' EIBRESP=' EIBRESP ' EIBRESP2=' EIBRESP2.
 
           IF BSDCOBCT-TERM-FLAG = 'X'
               DISPLAY 'BSDCOBCS TERMINATE REQUESTED'
               GO TO 50000-CLOSE-TCP
           END-IF.
 
           IF RETURN-CODE IS LESS THAN ZERO
               DISPLAY 'BSDCOBCS CONNECTION FAILED'
               GO TO 50000-CLOSE-TCP
           END-IF.
 
           IF TCPIP-MASK-READ = LOW-VALUES
               DISPLAY 'BSDCOBCS TIMEOUT EXPIRED'
               GO TO 30000-SELECTEX
           END-IF.
 
      ******************************************************************
      *
      * * BSD-C RECV() FUNCTION CALL...
      *
      *    THIS SAMPLE ASSUMES THAT THE COMPLETE INPUT MESSAGE IS
      *    FOUND IN A SINGLE RECEIVE. IN REALITY, YOU WILL NEED TO
      *    DETERMINE IF THE ENTIRE MESSAGE HAS BEEN RECEIVED, AND
      *    WILL MOST LIKELY NEED TO DO SOME FORM OF RECEIVE LOOPING
      *    TO GET THE ENTIRE MESSAGE.
      *
           DISPLAY 'BSDCOBCS CALLING IPNRRECV, SOCKET='
                   TCPIP-ACCEPT-SOCKET.
           CALL 'IPNRRECV' USING TCPIP-ACCEPT-SOCKET,
                                 TCPIP-RECVBUFF,
                                 TCPIP-RECVBUFF-LEN.
           MOVE RETURN-CODE TO TCPIP-LEN.
           MOVE RETURN-CODE TO DISPLAY-RC.
           DISPLAY 'BSDCOBCS RC=' DISPLAY-RC
                   ' RETCODE=' BSDCOBCT-RETCODE
                   ' ERRNO=' BSDCOBCT-ERRNO
                   ' EIBRESP=' EIBRESP ' EIBRESP2=' EIBRESP2.
 
           IF RETURN-CODE IS NOT GREATER THAN ZERO
               GO TO 50000-CLOSE-TCP.
 
           DISPLAY TCPIP-RECVBUFF(1:TCPIP-LEN).
 
      ******************************************************************
      *
      *    PREPARE REPLY DATA
      *
           MOVE 'HARTBEAT IS OK' TO TCPIP-SENDBUFF.
           DISPLAY TCPIP-SENDBUFF.
           MOVE X'15'      TO TCPIP-SENDBUFF(15:1).
 
      ******************************************************************
      *
      * * BSD-C SEND FUNCTION CALL...
      *
           DISPLAY 'BSDCOBCS CALLING IPNRSEND, SOCKET='
                   TCPIP-ACCEPT-SOCKET.
           CALL 'IPNRSEND' USING TCPIP-ACCEPT-SOCKET,
                                 TCPIP-SENDBUFF,
                                 TCPIP-SENDBUFF-LEN.
           MOVE RETURN-CODE TO DISPLAY-RC.
           DISPLAY 'BSDCOBCS RC=' DISPLAY-RC
                   ' RETCODE=' BSDCOBCT-RETCODE
                   ' ERRNO=' BSDCOBCT-ERRNO
                   ' EIBRESP=' EIBRESP ' EIBRESP2=' EIBRESP2.
 
           IF RETURN-CODE IS LESS THAN ZERO
               GO TO 50000-CLOSE-TCP.
 
           GO TO 30000-SELECTEX.
 
       50000-CLOSE-TCP.
 
      ******************************************************************
      *
      * * BSD-C CLOSE FUNCTION CALL... FOR ACCEPT SOCKET
      *
           DISPLAY 'BSDCOBCS CALLING IPNRCLOS, SOCKET='
                   TCPIP-ACCEPT-SOCKET.
           CALL 'IPNRCLOS' USING TCPIP-ACCEPT-SOCKET.
           MOVE RETURN-CODE TO DISPLAY-RC.
           DISPLAY 'BSDCOBCS RC=' DISPLAY-RC
                   ' RETCODE=' BSDCOBCT-RETCODE
                   ' ERRNO=' BSDCOBCT-ERRNO
                   ' EIBRESP=' EIBRESP ' EIBRESP2=' EIBRESP2.
      *
       90000-EXIT-CICS.
           DISPLAY 'BSDCOBCS EXIT'.
           MOVE ZERO TO RETURN-CODE.
           EXEC CICS RETURN END-EXEC.                                   00242000
