       IDENTIFICATION DIVISION.
       PROGRAM-ID.    BSDCOBCX.
       AUTHOR.        CSI.
       DATE-WRITTEN.  NOV 2007.
      ******************************************************************
      *                                                                *
      * THIS SOURCE MODULE IS PART OF A SET OF MODULES THAT REPRESENT  *
      * A SAMPLE BSD/C SERVER THAT RUNS IN CICS.                       *
      *                                                                *
      * THE SET OF MODULES INCLUDE THE FOLLOWING:                      *
      *                                                                *
      *  BSDCOBCL - MAIN LISTEN TASK                                   *
      *  BSDCOBCS - SERVER TASK THAT PROCESSES THE CONNECTED TASK(S)   *
      *  BSDCOBCX - TASK TO START/STOP THE LISTEN AND SERVER TASKS     *
      *  BSDCOBCT - TABLE TO CONTAIN COMMON FIELDS                     *
      *                                                                *
      *  THE REQUIRED CICS DEFINITIONS CAN BE FOUND IN MODULE BSDCOBCL *
      *                                                                *
      *  BSDCOBCX                                                      *
      *                                                                *
      *  THIS PROGRAM IS USED TO START THE MAIN LISTEN TASK OR TO      *
      *  STOP BOTH THE LISTEN TASK AND THE SERVER TASK.                *
      *                                                                *
      *  IT IS WRITTEN TO RUN FROM A CICS TERMINAL, BUT COULD BE       *
      *  EASILY MODIFIED TO RUN FROM THE CICS STARTUP AND SHUTDOWN     *
      *  PLT.                                                          *
      *                                                                *
      *  TTTT START                                                    *
      *  TTTT STOP                                                     *
      *                                                                *
      *  (TTTT) IS THE DEFINED CICS TRANID.                            *
      *                                                                *
      ******************************************************************
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       DATA DIVISION.
       WORKING-STORAGE SECTION.
 
       01  WS-EYE PIC X(24) VALUE 'BSDCOBCX-WORKING-STORAGE'.
       01  WS-MESSAGES.
           05  WS-MSG-STOPPED      PIC X(20) VALUE 'SERVER STOPPED'.
           05  WS-MSG-STARTED      PIC X(20) VALUE 'SERVER STARTED'.
           05  WS-MSG-INVREQ       PIC X(40) VALUE
                               'MUST SPECIFIY START OR STOP'.
 
       01  MISC-WS.
           05  WS-TIOA                  PIC X(20).
 
       LINKAGE SECTION.
 
      ******************************************************************
      *                                                                *
      *    BSDCOBCT TABLE  - LOADED FROM BSDCOBCT                      *
      *                                                                *
      *    THE FORMAT OF THIS TABLE IS KNOWN BY PROGRAMS:              *
      *                                                                *
      *    BSDCOBCL - SAMPLE COBOL BSD/C SERVER LISTEN TASK            *
      *    BSDCOBCS - SAMPLE COBOL BSD/C SERVER PROCESS TASK           *
      *    BSDCOBCX - SAMPLE COBOL BSD/C SERVER TERMINATION TASK       *
      *                                                                *
      ******************************************************************
 
       01  BSDCOBCT-TABLE.
           05  BSDCOBCT-EYECATCHER          PIC X(16).
           05  BSDCOBCT-ERRNO               PIC S9(8) COMP.                     N19990622 16:54:09
           05  BSDCOBCT-RETCODE             PIC S9(8) COMP.                     N19990622 16:54:09
      *    ECB MUST START ON FULL-WORD
           05  BSDCOBCT-TERM-ECB            PIC X(4).
           05  BSDCOBCT-TERM-FLAG           PIC X.
      ******************************************************************
      *                                                                *
      *    START OF PROCEDURE DIVISION.                                *
      *                                                                *
      ******************************************************************
 
       PROCEDURE DIVISION.
           DISPLAY 'BSDCOBCX ENTRY'.
      *
      *    LOAD COMMON CONTROL TABLE
      *
           EXEC CICS LOAD PROGRAM('BSDCOBCT') NOHANDLE
                              SET(ADDRESS OF BSDCOBCT-TABLE) END-EXEC.
 
           IF EIBRESP IS NOT = ZERO
               DISPLAY 'BSDCOBCX LOAD OF BSDCOBCT FAILED, RC='
                       EIBRESP
               GO TO 99999-RETURN
           END-IF.
 
           EXEC CICS RECEIVE INTO(WS-TIOA) END-EXEC.
           DISPLAY WS-TIOA.
 
           IF WS-TIOA(6:4) = 'STOP'
               MOVE 'X' TO BSDCOBCT-TERM-FLAG
               MOVE HIGH-VALUES TO BSDCOBCT-TERM-ECB
               EXEC CICS SEND FROM(WS-MSG-STOPPED)  ERASE END-EXEC
               DISPLAY 'BSDCOBCX  STOPPED'
               GO TO 99999-RETURN
           END-IF.
 
           IF WS-TIOA(6:5) = 'START'
               MOVE SPACE TO BSDCOBCT-TERM-FLAG
               MOVE LOW-VALUES TO BSDCOBCT-TERM-ECB
               EXEC CICS START TRANSID('BSDL') INTERVAL(0) END-EXEC
               EXEC CICS SEND FROM(WS-MSG-STARTED)  ERASE END-EXEC
               DISPLAY 'BSDCOBCX  STARTED'
               GO TO 99999-RETURN
           END-IF.
 
           EXEC CICS SEND FROM(WS-MSG-INVREQ) ERASE END-EXEC.
 
       99999-RETURN.
           DISPLAY 'BSDCOBCX EXIT'.
           MOVE ZERO TO RETURN-CODE.
           EXEC CICS RETURN END-EXEC.                                   00242000
